@@ -15,17 +15,18 @@ import shutil
 import tempfile
 from unittest import mock
 
-from ironic_lib import disk_partitioner
-from ironic_lib import disk_utils
 from ironic_lib import exception
 from ironic_lib import utils
 from oslo_concurrency import processutils
 from oslo_config import cfg
 import requests
 
+from ironic_python_agent import disk_partitioner
+from ironic_python_agent import disk_utils
 from ironic_python_agent import errors
 from ironic_python_agent import hardware
 from ironic_python_agent import partition_utils
+from ironic_python_agent import qemu_img
 from ironic_python_agent.tests.unit import base
 
 
@@ -167,10 +168,10 @@ class GetLabelledPartitionTestCases(base.IronicAgentTest):
                                                         self.node_uuid)
         self.assertEqual(part_result, result)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
+            mock.call('partprobe', self.dev, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
-                      use_standard_locale=True, run_as_root=True)
+                      use_standard_locale=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
 
@@ -183,10 +184,10 @@ class GetLabelledPartitionTestCases(base.IronicAgentTest):
                                                         self.node_uuid)
         self.assertEqual(part_result, result)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
+            mock.call('partprobe', self.dev, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
-                      use_standard_locale=True, run_as_root=True)
+                      use_standard_locale=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
 
@@ -198,10 +199,10 @@ class GetLabelledPartitionTestCases(base.IronicAgentTest):
                                                         self.node_uuid)
         self.assertIsNone(result)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
+            mock.call('partprobe', self.dev, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
-                      use_standard_locale=True, run_as_root=True)
+                      use_standard_locale=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
 
@@ -217,10 +218,10 @@ class GetLabelledPartitionTestCases(base.IronicAgentTest):
                                self.dev, self.config_part_label,
                                self.node_uuid)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
+            mock.call('partprobe', self.dev, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
-                      use_standard_locale=True, run_as_root=True)
+                      use_standard_locale=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
 
@@ -233,10 +234,10 @@ class GetLabelledPartitionTestCases(base.IronicAgentTest):
                                self.dev, self.config_part_label,
                                self.node_uuid)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
+            mock.call('partprobe', self.dev, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
-                      use_standard_locale=True, run_as_root=True)
+                      use_standard_locale=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
         self.assertEqual(1, mock_log.call_count)
@@ -253,7 +254,7 @@ class IsDiskLargerThanMaxSizeTestCases(base.IronicAgentTest):
         result = partition_utils._is_disk_larger_than_max_size(self.dev,
                                                                self.node_uuid)
         mock_execute.assert_called_once_with('blockdev', '--getsize64',
-                                             '/dev/fake', run_as_root=True,
+                                             '/dev/fake',
                                              use_standard_locale=True)
         return result
 
@@ -277,7 +278,7 @@ class IsDiskLargerThanMaxSizeTestCases(base.IronicAgentTest):
                                partition_utils._is_disk_larger_than_max_size,
                                self.dev, self.node_uuid)
         mock_execute.assert_called_once_with('blockdev', '--getsize64',
-                                             '/dev/fake', run_as_root=True,
+                                             '/dev/fake',
                                              use_standard_locale=True)
         self.assertEqual(1, mock_log.call_count)
 
@@ -452,13 +453,15 @@ class WorkOnDiskTestCase(base.IronicAgentTest):
     @mock.patch.object(utils, 'mkfs', lambda fs, path, label=None: None)
     @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
     @mock.patch.object(disk_utils, 'populate_image', lambda image_path,
-                       root_path, conv_flags=None: None)
+                       root_path, conv_flags=None, source_format=None,
+                       is_raw=False: None)
     def test_gpt_disk_label(self):
         ephemeral_part = '/dev/fake-part1'
         swap_part = '/dev/fake-part2'
         root_part = '/dev/fake-part3'
         ephemeral_mb = 256
         ephemeral_format = 'exttest'
+        source_format = 'raw'
 
         self.mock_mp.return_value = {'ephemeral': ephemeral_part,
                                      'swap': swap_part,
@@ -471,7 +474,8 @@ class WorkOnDiskTestCase(base.IronicAgentTest):
                                      self.swap_mb, ephemeral_mb,
                                      ephemeral_format,
                                      self.image_path, self.node_uuid,
-                                     disk_label='gpt', conv_flags=None)
+                                     disk_label='gpt', conv_flags=None,
+                                     source_format=source_format, is_raw=True)
         self.assertEqual(self.mock_ibd.call_args_list, calls)
         self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
                                              self.swap_mb, ephemeral_mb,
@@ -491,6 +495,8 @@ class WorkOnDiskTestCase(base.IronicAgentTest):
         """Test that we create a fat filesystem with UEFI localboot."""
         root_part = '/dev/fake-part1'
         efi_part = '/dev/fake-part2'
+        source_format = 'format'
+
         self.mock_mp.return_value = {'root': root_part,
                                      'efi system partition': efi_part}
         self.mock_ibd.return_value = True
@@ -501,7 +507,8 @@ class WorkOnDiskTestCase(base.IronicAgentTest):
                                      self.swap_mb, self.ephemeral_mb,
                                      self.ephemeral_format,
                                      self.image_path, self.node_uuid,
-                                     boot_mode="uefi")
+                                     boot_mode="uefi",
+                                     source_format=source_format, is_raw=False)
 
         self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
                                              self.swap_mb, self.ephemeral_mb,
@@ -514,8 +521,9 @@ class WorkOnDiskTestCase(base.IronicAgentTest):
         self.assertEqual(self.mock_ibd.call_args_list, mock_ibd_calls)
         mock_mkfs.assert_called_once_with(fs='vfat', path=efi_part,
                                           label='efi-part')
-        mock_populate_image.assert_called_once_with(self.image_path,
-                                                    root_part, conv_flags=None)
+        mock_populate_image.assert_called_once_with(
+            self.image_path, root_part, conv_flags=None,
+            source_format=source_format, is_raw=False)
         mock_block_uuid.assert_any_call(root_part)
         mock_block_uuid.assert_any_call(efi_part)
         mock_trigger_device_rescan.assert_called_once_with(self.dev)
@@ -594,6 +602,7 @@ class WorkOnDiskTestCase(base.IronicAgentTest):
         root_part = '/dev/fake-part3'
         ephemeral_mb = 256
         ephemeral_format = 'exttest'
+        fmt = 'format'
 
         self.mock_mp.return_value = {'ephemeral': ephemeral_part,
                                      'swap': swap_part,
@@ -603,11 +612,15 @@ class WorkOnDiskTestCase(base.IronicAgentTest):
                                      self.swap_mb, ephemeral_mb,
                                      ephemeral_format,
                                      self.image_path, self.node_uuid,
-                                     disk_label='gpt', conv_flags='sparse')
+                                     disk_label='gpt', conv_flags='sparse',
+                                     source_format=fmt,
+                                     is_raw=False)
 
         mock_populate_image.assert_called_once_with(self.image_path,
                                                     root_part,
-                                                    conv_flags='sparse')
+                                                    conv_flags='sparse',
+                                                    source_format=fmt,
+                                                    is_raw=False)
 
 
 class CreateConfigDriveTestCases(base.IronicAgentTest):
@@ -697,12 +710,12 @@ class CreateConfigDriveTestCases(base.IronicAgentTest):
                                                       config_url)
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', '-u', '0:fake-uuid',
-                      self.dev, run_as_root=True),
+                      self.dev),
             mock.call('sync'),
             mock.call('udevadm', 'settle'),
-            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
-            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
-
+            mock.call('partprobe', self.dev, attempts=10),
+            mock.call('udevadm', 'settle'),
+            mock.call('sgdisk', '-v', self.dev),
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       delay_on_retry=True)
@@ -758,11 +771,12 @@ class CreateConfigDriveTestCases(base.IronicAgentTest):
                                                       config_url)
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', '-u', '0:fake-uuid',
-                      self.dev, run_as_root=True),
+                      self.dev),
             mock.call('sync'),
             mock.call('udevadm', 'settle'),
-            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
-            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, attempts=10),
+            mock.call('udevadm', 'settle'),
+            mock.call('sgdisk', '-v', self.dev),
 
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
@@ -823,12 +837,12 @@ class CreateConfigDriveTestCases(base.IronicAgentTest):
                                                       config_url)
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', '-u', '0:fake-uuid',
-                      self.dev, run_as_root=True),
+                      self.dev),
             mock.call('sync'),
             mock.call('udevadm', 'settle'),
-            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
-            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
-
+            mock.call('partprobe', self.dev, attempts=10),
+            mock.call('udevadm', 'settle'),
+            mock.call('sgdisk', '-v', self.dev),
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       delay_on_retry=True)
@@ -918,19 +932,20 @@ class CreateConfigDriveTestCases(base.IronicAgentTest):
             parted_call = mock.call('parted', '-a', 'optimal', '-s',
                                     '--', self.dev, 'mkpart',
                                     'primary', 'fat32', 2097087,
-                                    2097151, run_as_root=True)
+                                    2097151)
         else:
             self.assertEqual(0, mock_log.call_count)
             parted_call = mock.call('parted', '-a', 'optimal', '-s',
                                     '--', self.dev, 'mkpart',
                                     'primary', 'fat32', '-64MiB',
-                                    '-0', run_as_root=True)
+                                    '-0')
         mock_execute.assert_has_calls([
             parted_call,
             mock.call('sync'),
             mock.call('udevadm', 'settle'),
-            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
-            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, attempts=10),
+            mock.call('udevadm', 'settle'),
+            mock.call('sgdisk', '-v', self.dev),
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       delay_on_retry=True)
@@ -1025,12 +1040,12 @@ class CreateConfigDriveTestCases(base.IronicAgentTest):
         mock_execute.assert_has_calls([
             mock.call('parted', '-a', 'optimal', '-s', '--',
                       self.dev, 'mkpart', 'primary',
-                      'fat32', '-64MiB', '-0',
-                      run_as_root=True),
+                      'fat32', '-64MiB', '-0'),
             mock.call('sync'),
             mock.call('udevadm', 'settle'),
-            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
-            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, attempts=10),
+            mock.call('udevadm', 'settle'),
+            mock.call('sgdisk', '-v', self.dev),
         ])
 
         self.assertEqual(2, mock_list_partitions.call_count)
@@ -1096,8 +1111,7 @@ class CreateConfigDriveTestCases(base.IronicAgentTest):
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         mock_execute.assert_called_with('parted', '-a', 'optimal', '-s', '--',
                                         self.dev, 'mkpart', 'primary',
-                                        'fat32', '-64MiB', '-0',
-                                        run_as_root=True)
+                                        'fat32', '-64MiB', '-0')
         self.assertEqual(1, mock_list_partitions.call_count)
         mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_table_type.assert_called_with(self.dev)
@@ -1224,12 +1238,14 @@ class CreateConfigDriveTestCases(base.IronicAgentTest):
 # NOTE(TheJulia): trigger_device_rescan is systemwide thus pointless
 # to execute in the file test case. Also, CI unit test jobs lack sgdisk.
 @mock.patch.object(disk_utils, 'trigger_device_rescan', autospec=True)
-@mock.patch.object(utils, 'wait_for_disk_to_become_available', autospec=True)
+@mock.patch.object(disk_utils, 'wait_for_disk_to_become_available',
+                   autospec=True)
 @mock.patch.object(disk_utils, 'is_block_device', autospec=True)
 @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
 @mock.patch.object(disk_utils, 'dd', autospec=True)
-@mock.patch.object(disk_utils, 'convert_image', autospec=True)
+@mock.patch.object(qemu_img, 'convert_image', autospec=True)
 @mock.patch.object(utils, 'mkfs', autospec=True)
+@mock.patch.object(disk_utils, 'populate_image', autospec=True)
 # NOTE(dtantsur): destroy_disk_metadata resets file size, disabling it
 @mock.patch.object(disk_utils, 'destroy_disk_metadata', autospec=True)
 class RealFilePartitioningTestCase(base.IronicAgentTest):
@@ -1257,48 +1273,33 @@ class RealFilePartitioningTestCase(base.IronicAgentTest):
         utils.execute('dd', 'if=/dev/zero', 'of=%s' % self.file.name,
                       'bs=1', 'count=0', 'seek=20MiB')
 
-    @staticmethod
-    def _run_without_root(func, *args, **kwargs):
-        """Make sure root is not required when using utils.execute."""
-        real_execute = utils.execute
-
-        def fake_execute(*cmd, **kwargs):
-            kwargs['run_as_root'] = False
-            return real_execute(*cmd, **kwargs)
-
-        with mock.patch.object(utils, 'execute', fake_execute):
-            return func(*args, **kwargs)
-
-    def test_different_sizes(self, mock_destroy, mock_mkfs, mock_convert,
-                             mock_dd, mock_block_uuid, mock_is_block,
-                             mock_wait, mock_trigger_rescan):
+    def test_different_sizes(self, mock_destroy, mock_populate, mock_mkfs,
+                             mock_convert, mock_dd, mock_block_uuid,
+                             mock_is_block, mock_wait, mock_trigger_rescan):
         # NOTE(dtantsur): Keep this list in order with expected partitioning
         fields = ['ephemeral_mb', 'swap_mb', 'root_mb']
         variants = ((0, 0, 12), (4, 2, 8), (0, 4, 10), (5, 0, 10))
         for variant in variants:
             kwargs = dict(zip(fields, variant))
-            self._run_without_root(partition_utils.work_on_disk,
-                                   self.file.name, ephemeral_format='ext4',
-                                   node_uuid='', image_path='path', **kwargs)
-            part_table = self._run_without_root(
-                disk_utils.list_partitions, self.file.name)
+            partition_utils.work_on_disk(
+                self.file.name, ephemeral_format='ext4',
+                node_uuid='', image_path='path', **kwargs)
+            part_table = disk_utils.list_partitions(self.file.name)
             for part, expected_size in zip(part_table, filter(None, variant)):
                 self.assertEqual(expected_size, part['size'],
                                  "comparison failed for %s" % list(variant))
 
-    def test_whole_disk(self, mock_destroy, mock_mkfs, mock_convert, mock_dd,
-                        mock_block_uuid, mock_is_block, mock_wait,
-                        mock_trigger_rescan):
+    def test_whole_disk(self, mock_destroy, mock_populate, mock_mkfs,
+                        mock_convert, mock_dd, mock_block_uuid,
+                        mock_is_block, mock_wait, mock_trigger_rescan):
         # 6 MiB ephemeral + 3 MiB swap + 9 MiB root + 1 MiB for MBR
         # + 1 MiB MAGIC == 20 MiB whole disk
         # TODO(dtantsur): figure out why we need 'magic' 1 more MiB
         # and why the is different on Ubuntu and Fedora (see below)
-        self._run_without_root(partition_utils.work_on_disk, self.file.name,
-                               root_mb=9, ephemeral_mb=6, swap_mb=3,
-                               ephemeral_format='ext4', node_uuid='',
-                               image_path='path')
-        part_table = self._run_without_root(
-            disk_utils.list_partitions, self.file.name)
+        partition_utils.work_on_disk(
+            self.file.name, root_mb=9, ephemeral_mb=6, swap_mb=3,
+            ephemeral_format='ext4', node_uuid='', image_path='path')
+        part_table = disk_utils.list_partitions(self.file.name)
         sizes = [part['size'] for part in part_table]
         # NOTE(dtantsur): parted in Ubuntu 12.04 will occupy the last MiB,
         # parted in Fedora 20 won't - thus two possible variants for last part
@@ -1501,7 +1502,7 @@ class TestConfigDriveTestRecovery(base.IronicAgentTest):
         mock_mkfs.side_effect = processutils.ProcessExecutionError('boom')
         self.assertRaisesRegex(
             exception.InstanceDeployFailure,
-            'A failure occured while attempting to format.*',
+            'A failure occurred while attempting to format.*',
             partition_utils._try_build_fat32_config_drive,
             self.fake_dev,
             self.configdrive_file)
