@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import base64
 import errno
 import glob
 import io
@@ -21,12 +22,11 @@ import shutil
 import subprocess
 import tarfile
 import tempfile
+import time
 from unittest import mock
 
-from ironic_lib import disk_utils
 from ironic_lib import utils as ironic_utils
 from oslo_concurrency import processutils
-from oslo_serialization import base64
 import requests
 import testtools
 
@@ -35,18 +35,6 @@ from ironic_python_agent import hardware
 from ironic_python_agent.tests.unit import base as ironic_agent_base
 from ironic_python_agent import utils
 
-PARTED_OUTPUT_UNFORMATTED = '''Model: whatever
-Disk /dev/sda: 450GB
-Sector size (logical/physical): 512B/512B
-Partition Table: {}
-Disk Flags:
-
-Number  Start   End     Size    File system  Name  Flags
-14      1049kB  5243kB  4194kB                     bios_grub
-15      5243kB  116MB   111MB   fat32              boot, esp
- 1      116MB   2361MB  2245MB  ext4
-'''
-
 
 class ExecuteTestCase(ironic_agent_base.IronicAgentTest):
     # This test case does call utils.execute(), so don't block access to the
@@ -62,6 +50,79 @@ class ExecuteTestCase(ironic_agent_base.IronicAgentTest):
                                              check_exit_code=False)
 
 
+@mock.patch('shutil.rmtree', autospec=True)
+@mock.patch.object(ironic_utils, 'execute', autospec=True)
+@mock.patch('tempfile.mkdtemp', autospec=True)
+class MountedTestCase(ironic_agent_base.IronicAgentTest):
+
+    def test_temporary(self, mock_temp, mock_execute, mock_rmtree):
+        with utils.mounted('/dev/fake') as path:
+            self.assertIs(path, mock_temp.return_value)
+        mock_execute.assert_has_calls([
+            mock.call("mount", '/dev/fake', mock_temp.return_value,
+                      attempts=1, delay_on_retry=True),
+            mock.call("umount", mock_temp.return_value,
+                      attempts=3, delay_on_retry=True),
+        ])
+        mock_rmtree.assert_called_once_with(mock_temp.return_value)
+
+    def test_with_dest(self, mock_temp, mock_execute, mock_rmtree):
+        with utils.mounted('/dev/fake', '/mnt/fake') as path:
+            self.assertEqual('/mnt/fake', path)
+        mock_execute.assert_has_calls([
+            mock.call("mount", '/dev/fake', '/mnt/fake',
+                      attempts=1, delay_on_retry=True),
+            mock.call("umount", '/mnt/fake',
+                      attempts=3, delay_on_retry=True),
+        ])
+        self.assertFalse(mock_temp.called)
+        self.assertFalse(mock_rmtree.called)
+
+    def test_with_opts(self, mock_temp, mock_execute, mock_rmtree):
+        with utils.mounted('/dev/fake', '/mnt/fake',
+                           opts=['ro', 'foo=bar']) as path:
+            self.assertEqual('/mnt/fake', path)
+        mock_execute.assert_has_calls([
+            mock.call("mount", '/dev/fake', '/mnt/fake', '-o', 'ro,foo=bar',
+                      attempts=1, delay_on_retry=True),
+            mock.call("umount", '/mnt/fake',
+                      attempts=3, delay_on_retry=True),
+        ])
+
+    def test_with_type(self, mock_temp, mock_execute, mock_rmtree):
+        with utils.mounted('/dev/fake', '/mnt/fake',
+                           fs_type='iso9660') as path:
+            self.assertEqual('/mnt/fake', path)
+        mock_execute.assert_has_calls([
+            mock.call("mount", '/dev/fake', '/mnt/fake', '-t', 'iso9660',
+                      attempts=1, delay_on_retry=True),
+            mock.call("umount", '/mnt/fake',
+                      attempts=3, delay_on_retry=True),
+        ])
+
+    def test_failed_to_mount(self, mock_temp, mock_execute, mock_rmtree):
+        mock_execute.side_effect = OSError
+        self.assertRaises(OSError, utils.mounted('/dev/fake').__enter__)
+        mock_execute.assert_called_once_with("mount", '/dev/fake',
+                                             mock_temp.return_value,
+                                             attempts=1,
+                                             delay_on_retry=True)
+        mock_rmtree.assert_called_once_with(mock_temp.return_value)
+
+    def test_failed_to_unmount(self, mock_temp, mock_execute, mock_rmtree):
+        mock_execute.side_effect = [('', ''),
+                                    processutils.ProcessExecutionError]
+        with utils.mounted('/dev/fake', '/mnt/fake') as path:
+            self.assertEqual('/mnt/fake', path)
+        mock_execute.assert_has_calls([
+            mock.call("mount", '/dev/fake', '/mnt/fake',
+                      attempts=1, delay_on_retry=True),
+            mock.call("umount", '/mnt/fake',
+                      attempts=3, delay_on_retry=True),
+        ])
+        self.assertFalse(mock_rmtree.called)
+
+
 class GetAgentParamsTestCase(ironic_agent_base.IronicAgentTest):
 
     @mock.patch('oslo_log.log.getLogger', autospec=True)
@@ -199,7 +260,7 @@ class GetAgentParamsTestCase(ironic_agent_base.IronicAgentTest):
     @mock.patch.object(utils, '_check_vmedia_device', autospec=True)
     @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
     @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(ironic_utils, 'mounted', autospec=True)
+    @mock.patch.object(utils, 'mounted', autospec=True)
     def test__get_vmedia_params(self, mount_mock, read_params_mock, find_mock,
                                 check_vmedia_mock):
         check_vmedia_mock.return_value = True
@@ -218,7 +279,7 @@ class GetAgentParamsTestCase(ironic_agent_base.IronicAgentTest):
     @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
     @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
     @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(ironic_utils, 'mounted', autospec=True)
+    @mock.patch.object(utils, 'mounted', autospec=True)
     def test__get_vmedia_params_by_device(self, mount_mock, read_params_mock,
                                           get_device_mock, find_mock,
                                           check_vmedia_mock):
@@ -240,7 +301,7 @@ class GetAgentParamsTestCase(ironic_agent_base.IronicAgentTest):
     @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
     @mock.patch.object(utils, '_get_vmedia_device', autospec=True)
     @mock.patch.object(utils, '_read_params_from_file', autospec=True)
-    @mock.patch.object(ironic_utils, 'mounted', autospec=True)
+    @mock.patch.object(utils, 'mounted', autospec=True)
     def test__get_vmedia_params_by_device_device_invalid(
             self, mount_mock, read_params_mock,
             get_device_mock, find_mock,
@@ -291,7 +352,7 @@ class TestFailures(testtools.TestCase):
         self.assertRaisesRegex(FakeException, 'foo', f.raise_if_needed)
 
 
-class TestUtils(testtools.TestCase):
+class TestUtils(ironic_agent_base.IronicAgentTest):
 
     def _get_journalctl_output(self, mock_execute, lines=None, units=None):
         contents = b'Krusty Krab'
@@ -333,7 +394,7 @@ class TestUtils(testtools.TestCase):
         data = utils.gzip_and_b64encode(io_dict=io_dict)
         self.assertIsInstance(data, str)
 
-        res = io.BytesIO(base64.decode_as_bytes(data))
+        res = io.BytesIO(base64.b64decode(data))
         with tarfile.open(fileobj=res) as tar:
             members = [(m.name, m.size) for m in tar]
             self.assertEqual([('fake-name', len(contents))], members)
@@ -424,11 +485,11 @@ class TestUtils(testtools.TestCase):
         self.assertFalse(utils.is_journalctl_present())
 
     @mock.patch.object(utils, 'gzip_and_b64encode', autospec=True)
+    @mock.patch.object(hardware, 'dispatch_to_all_managers', autospec=True)
     @mock.patch.object(utils, 'is_journalctl_present', autospec=True)
-    @mock.patch.object(utils, 'get_command_output', autospec=True)
     @mock.patch.object(utils, 'get_journalctl_output', autospec=True)
     def test_collect_system_logs_journald(
-            self, mock_logs, mock_outputs, mock_journalctl, mock_gzip_b64):
+            self, mock_logs, mock_journalctl, mock_dispatch, mock_gzip_b64):
         mock_journalctl.return_value = True
         ret = 'Patrick Star'
         mock_gzip_b64.return_value = ret
@@ -436,43 +497,68 @@ class TestUtils(testtools.TestCase):
         logs_string = utils.collect_system_logs()
         self.assertEqual(ret, logs_string)
         mock_logs.assert_called_once_with(lines=None)
-        calls = [mock.call(['ps', 'au']), mock.call(['df', '-a']),
-                 mock.call(['iptables', '-L']), mock.call(['ip', 'addr']),
-                 mock.call(['lshw', '-quiet', '-json'])]
-        mock_outputs.assert_has_calls(calls, any_order=True)
         mock_gzip_b64.assert_called_once_with(
-            io_dict={'journal': mock.ANY, 'ps': mock.ANY, 'df': mock.ANY,
-                     'iptables': mock.ANY, 'ip_addr': mock.ANY,
-                     'lshw': mock.ANY, 'lsblk': mock.ANY,
-                     'lsblk-full': mock.ANY, 'mdstat': mock.ANY,
-                     'mount': mock.ANY, 'parted': mock.ANY,
-                     'multipath': mock.ANY},
-            file_list=[])
+            io_dict=mock.ANY, file_list=[])
+        mock_dispatch.assert_called_once_with('collect_system_logs',
+                                              mock.ANY, [])
 
     @mock.patch.object(utils, 'gzip_and_b64encode', autospec=True)
+    @mock.patch.object(hardware, 'dispatch_to_all_managers', autospec=True)
+    @mock.patch.object(utils, 'is_journalctl_present', autospec=True)
+    @mock.patch.object(utils, 'get_journalctl_output', autospec=True)
+    def test_collect_system_logs_journald_with_logfile(
+            self, mock_logs, mock_journalctl, mock_dispatch, mock_gzip_b64):
+        tmp = tempfile.NamedTemporaryFile()
+        self.addCleanup(lambda: tmp.close())
+
+        self.config(log_file=tmp.name)
+        mock_journalctl.return_value = True
+        ret = 'Patrick Star'
+        mock_gzip_b64.return_value = ret
+
+        logs_string = utils.collect_system_logs()
+        self.assertEqual(ret, logs_string)
+        mock_logs.assert_called_once_with(lines=None)
+        mock_gzip_b64.assert_called_once_with(
+            io_dict=mock.ANY, file_list=[tmp.name])
+        mock_dispatch.assert_called_once_with('collect_system_logs',
+                                              mock.ANY, [tmp.name])
+
+    @mock.patch.object(utils, 'gzip_and_b64encode', autospec=True)
+    @mock.patch.object(hardware, 'dispatch_to_all_managers', autospec=True)
     @mock.patch.object(utils, 'is_journalctl_present', autospec=True)
-    @mock.patch.object(utils, 'get_command_output', autospec=True)
     def test_collect_system_logs_non_journald(
-            self, mock_outputs, mock_journalctl, mock_gzip_b64):
+            self, mock_journalctl, mock_dispatch, mock_gzip_b64):
         mock_journalctl.return_value = False
         ret = 'SpongeBob SquarePants'
         mock_gzip_b64.return_value = ret
 
         logs_string = utils.collect_system_logs()
         self.assertEqual(ret, logs_string)
-        calls = [mock.call(['dmesg']), mock.call(['ps', 'au']),
-                 mock.call(['df', '-a']), mock.call(['iptables', '-L']),
-                 mock.call(['ip', 'addr']),
-                 mock.call(['lshw', '-quiet', '-json'])]
-        mock_outputs.assert_has_calls(calls, any_order=True)
         mock_gzip_b64.assert_called_once_with(
-            io_dict={'dmesg': mock.ANY, 'ps': mock.ANY, 'df': mock.ANY,
-                     'iptables': mock.ANY, 'ip_addr': mock.ANY,
-                     'lshw': mock.ANY, 'lsblk': mock.ANY,
-                     'lsblk-full': mock.ANY, 'mdstat': mock.ANY,
-                     'mount': mock.ANY, 'parted': mock.ANY,
-                     'multipath': mock.ANY},
-            file_list=['/var/log'])
+            io_dict=mock.ANY, file_list=['/var/log'])
+        mock_dispatch.assert_called_once_with('collect_system_logs',
+                                              mock.ANY, ['/var/log'])
+
+    @mock.patch.object(utils, 'gzip_and_b64encode', autospec=True)
+    @mock.patch.object(hardware, 'dispatch_to_all_managers', autospec=True)
+    @mock.patch.object(utils, 'is_journalctl_present', autospec=True)
+    def test_collect_system_logs_non_journald_with_logfile(
+            self, mock_journalctl, mock_dispatch, mock_gzip_b64):
+        tmp = tempfile.NamedTemporaryFile()
+        self.addCleanup(lambda: tmp.close())
+
+        self.config(log_file=tmp.name)
+        mock_journalctl.return_value = False
+        ret = 'SpongeBob SquarePants'
+        mock_gzip_b64.return_value = ret
+
+        logs_string = utils.collect_system_logs()
+        self.assertEqual(ret, logs_string)
+        mock_gzip_b64.assert_called_once_with(
+            io_dict=mock.ANY, file_list=['/var/log', tmp.name])
+        mock_dispatch.assert_called_once_with('collect_system_logs',
+                                              mock.ANY, ['/var/log', tmp.name])
 
     def test_get_ssl_client_options(self):
         # defaults
@@ -771,36 +857,6 @@ class TestUtils(testtools.TestCase):
         self.assertEqual('gpt', label)
         mock_boot_mode.assert_has_calls([])
 
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test_scan_partition_table_type_gpt(self, mocked_execute):
-        self._test_scan_partition_table_by_type(mocked_execute, 'gpt', 'gpt')
-
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test_scan_partition_table_type_msdos(self, mocked_execute):
-        self._test_scan_partition_table_by_type(mocked_execute, 'msdos',
-                                                'msdos')
-
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test_scan_partition_table_type_unknown(self, mocked_execute):
-        self._test_scan_partition_table_by_type(mocked_execute, 'whatever',
-                                                'unknown')
-
-    def _test_scan_partition_table_by_type(self, mocked_execute,
-                                           table_type_output,
-                                           expected_table_type):
-
-        parted_ret = PARTED_OUTPUT_UNFORMATTED.format(table_type_output)
-
-        mocked_execute.side_effect = [
-            (parted_ret, None),
-        ]
-
-        ret = utils.scan_partition_table_type('hello')
-        mocked_execute.assert_has_calls(
-            [mock.call('parted', '-s', 'hello', '--', 'print')]
-        )
-        self.assertEqual(expected_table_type, ret)
-
 
 class TestRemoveKeys(testtools.TestCase):
     def test_remove_keys(self):
@@ -895,59 +951,19 @@ class TestClockSyncUtils(ironic_agent_base.IronicAgentTest):
         self.assertEqual(0, mock_execute.call_count)
 
 
-@mock.patch.object(disk_utils, 'list_partitions', autospec=True)
-@mock.patch.object(utils, 'scan_partition_table_type', autospec=True)
-class TestGetEfiPart(testtools.TestCase):
-
-    def test_get_efi_part_on_device(self, mocked_type, mocked_parts):
-        mocked_parts.return_value = [
-            {'number': '1', 'flags': ''},
-            {'number': '14', 'flags': 'bios_grub'},
-            {'number': '15', 'flags': 'esp, boot'},
-        ]
-        ret = utils.get_efi_part_on_device('/dev/sda')
-        self.assertEqual('15', ret)
-
-    def test_get_efi_part_on_device_only_boot_flag_gpt(self, mocked_type,
-                                                       mocked_parts):
-        mocked_type.return_value = 'gpt'
-        mocked_parts.return_value = [
-            {'number': '1', 'flags': ''},
-            {'number': '14', 'flags': 'bios_grub'},
-            {'number': '15', 'flags': 'boot'},
-        ]
-        ret = utils.get_efi_part_on_device('/dev/sda')
-        self.assertEqual('15', ret)
-
-    def test_get_efi_part_on_device_only_boot_flag_mbr(self, mocked_type,
-                                                       mocked_parts):
-        mocked_type.return_value = 'msdos'
-        mocked_parts.return_value = [
-            {'number': '1', 'flags': ''},
-            {'number': '14', 'flags': 'bios_grub'},
-            {'number': '15', 'flags': 'boot'},
-        ]
-        self.assertIsNone(utils.get_efi_part_on_device('/dev/sda'))
-
-    def test_get_efi_part_on_device_not_found(self, mocked_type, mocked_parts):
-        mocked_parts.return_value = [
-            {'number': '1', 'flags': ''},
-            {'number': '14', 'flags': 'bios_grub'},
-        ]
-        self.assertIsNone(utils.get_efi_part_on_device('/dev/sda'))
-
-
+@mock.patch.object(utils, '_unmount_any_config_drives', autospec=True)
 @mock.patch.object(utils, '_booted_from_vmedia', autospec=True)
 @mock.patch.object(utils, '_check_vmedia_device', autospec=True)
 @mock.patch.object(utils, '_find_vmedia_device_by_labels', autospec=True)
 @mock.patch.object(shutil, 'copy', autospec=True)
-@mock.patch.object(ironic_utils, 'mounted', autospec=True)
+@mock.patch.object(utils, 'mounted', autospec=True)
 @mock.patch.object(utils, 'execute', autospec=True)
 class TestCopyConfigFromVmedia(testtools.TestCase):
 
     def test_vmedia_found_not_booted_from_vmedia(
             self, mock_execute, mock_mount, mock_copy,
-            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia,
+            mock_unmount_config):
         mock_booted_from_vmedia.return_value = False
         mock_find_device.return_value = '/dev/fake'
         utils.copy_config_from_vmedia()
@@ -956,10 +972,12 @@ class TestCopyConfigFromVmedia(testtools.TestCase):
         mock_copy.assert_not_called()
         mock_check_vmedia.assert_not_called()
         self.assertTrue(mock_booted_from_vmedia.called)
+        self.assertTrue(mock_unmount_config.called)
 
     def test_no_vmedia(
             self, mock_execute, mock_mount, mock_copy,
-            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia,
+            mock_unmount_config):
         mock_booted_from_vmedia.return_value = True
         mock_find_device.return_value = None
         utils.copy_config_from_vmedia()
@@ -968,10 +986,12 @@ class TestCopyConfigFromVmedia(testtools.TestCase):
         mock_copy.assert_not_called()
         mock_check_vmedia.assert_not_called()
         self.assertFalse(mock_booted_from_vmedia.called)
+        self.assertTrue(mock_unmount_config.called)
 
     def test_no_files(
             self, mock_execute, mock_mount, mock_copy,
-            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia,
+            mock_unmount_config):
         mock_booted_from_vmedia.return_value = True
         temp_path = tempfile.mkdtemp()
         self.addCleanup(lambda: shutil.rmtree(temp_path))
@@ -985,10 +1005,12 @@ class TestCopyConfigFromVmedia(testtools.TestCase):
                                              '/dev/something')
         mock_copy.assert_not_called()
         self.assertTrue(mock_booted_from_vmedia.called)
+        self.assertTrue(mock_unmount_config.called)
 
     def test_mounted_no_files(
             self, mock_execute, mock_mount, mock_copy,
-            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia,
+            mock_unmount_config):
 
         mock_booted_from_vmedia.return_value = True
         mock_execute.return_value = '/some/path', ''
@@ -999,11 +1021,13 @@ class TestCopyConfigFromVmedia(testtools.TestCase):
         mock_copy.assert_not_called()
         mock_mount.assert_not_called()
         self.assertTrue(mock_booted_from_vmedia.called)
+        self.assertTrue(mock_unmount_config.called)
 
     @mock.patch.object(os, 'makedirs', autospec=True)
     def test_copy(
             self, mock_makedirs, mock_execute, mock_mount, mock_copy,
-            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia):
+            mock_find_device, mock_check_vmedia, mock_booted_from_vmedia,
+            mock_unmount_config):
 
         mock_booted_from_vmedia.return_value = True
         mock_find_device.return_value = '/dev/something'
@@ -1042,12 +1066,14 @@ class TestCopyConfigFromVmedia(testtools.TestCase):
             mock.call(mock.ANY, '/etc/ironic-python-agent.d/ironic.conf'),
         ], any_order=True)
         self.assertTrue(mock_booted_from_vmedia.called)
+        self.assertTrue(mock_unmount_config.called)
 
     @mock.patch.object(os, 'makedirs', autospec=True)
     def test_copy_mounted(
             self, mock_makedirs, mock_execute, mock_mount,
             mock_copy, mock_find_device, mock_check_vmedia,
-            mock_booted_from_vmedia):
+            mock_booted_from_vmedia,
+            mock_unmount_config):
         mock_booted_from_vmedia.return_value = True
         mock_find_device.return_value = '/dev/something'
         path = tempfile.mkdtemp()
@@ -1083,6 +1109,7 @@ class TestCopyConfigFromVmedia(testtools.TestCase):
         ], any_order=True)
         mock_mount.assert_not_called()
         self.assertTrue(mock_booted_from_vmedia.called)
+        self.assertTrue(mock_unmount_config.called)
 
 
 @mock.patch.object(requests, 'get', autospec=True)
@@ -1203,3 +1230,20 @@ class TestCheckEarlyLogging(ironic_agent_base.IronicAgentTest):
         expected_calls = [mock.call('Early logging: %s', 'line 1.'),
                           mock.call('Early logging: %s', 'line 2 message')]
         info.assert_has_calls(expected_calls)
+
+
+class TestUnmountOfConfig(ironic_agent_base.IronicAgentTest):
+
+    @mock.patch.object(utils, '_early_log', autospec=True)
+    @mock.patch.object(os.path, 'ismount', autospec=True)
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(time, 'sleep', autospec=True)
+    def test__unmount_any_config_drives(self, mock_sleep, mock_exec,
+                                        mock_ismount, mock_log,):
+        mock_ismount.side_effect = iter([True, True, False])
+        utils._unmount_any_config_drives()
+        self.assertEqual(2, mock_sleep.call_count)
+        self.assertEqual(2, mock_log.call_count)
+        mock_exec.assert_has_calls([
+            mock.call('umount', '/mnt/config'),
+            mock.call('umount', '/mnt/config')])
